#import "MyOpenGLView.h"
#import "NTBox.h"
#import "NTTank.h"
#import "NTBall.h"
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <GLUT/glut.h>

void  PhysicsApplyForceAndTorque (const NewtonBody* body);
void  TankApplyForceAndTorque (const NewtonBody* body);
void  PhysicsSetTransform (const NewtonBody* body, const float* matrix);

BOOL	gUp = NO, gDown = NO, gLeft = NO, gRight = NO, gShoot= NO;
GLuint floorTex = 0;
	int player;
	NewtonWorld* nWorld;
	NSMutableArray *boxes;
	
	int playerBoxes[2]={4,4};
	
	MyOpenGLView *view;

@implementation MyOpenGLView

- (id)initWithFrame:(NSRect)frameRect
{
	//NSOpenGLPixelFormatAttribute		attributes[]={NSOpenGLPFADepthSize, 16, nil};
	GLuint attributes[] = 
	{
		NSOpenGLPFAAccelerated,
		NSOpenGLPFADoubleBuffer,
		NSOpenGLPFANoRecovery,
		NSOpenGLPFAStencilSize, 8,
		NSOpenGLPFADepthSize, 16,
		0
	};

    // This next line calls our parent's init routine
    self = [super initWithFrame:frameRect pixelFormat:[[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes] autorelease]];
    
    [self setup];
	view = self;
	
	player = 1;
	
	// start timer
	[[NSTimer scheduledTimerWithTimeInterval: 0.0001 target: self selector:@selector(doTimeStep:) userInfo: nil repeats: YES] retain];
    
	//[NSThread detachNewThreadSelector:@selector( physicsThread: ) toTarget:self withObject:nil];
	
    return self;
}

- (void)awakeFromNib
{
	[NSApp setDelegate:self];
	//[[self window] makeKeyAndOrderFront];
}

- (void) viewDidMoveToWindow
{
	[super viewDidMoveToWindow];
	[[self window] setOpaque:NO];
	[[self window] setAlphaValue:.999f];
}
	
- (void)setup
{
	[self setupGL];
	
	// create the newton world
	nWorld = NewtonCreate (NULL, NULL);

	// set the linear solver model for faster speed 
	NewtonSetSolverModel (nWorld, 8);

	// set the adpative friction model for faster speed 
	//NewtonSetFrictionModel (nWorld, 1);
	[self initScene];
}

- (void)setupGL
{
	// initilize opengl	
    [[self openGLContext] makeCurrentContext];
	
	long order = -1;
	[[self openGLContext] setValues:&order forParameter: NSOpenGLCPSurfaceOrder];
	
	floorTex = GetOpenGLTextureFromPath([NSString stringWithFormat:@"%@/Walker.png", [[NSBundle mainBundle] resourcePath]]);
	
	// set the background color
	glClearColor(0.29, 0.42, 0.58, 0.0);
	
	// set up a single white light
	GLfloat lightColor[] = { 0.7f, 0.7f, 0.7f, 0.0 };
	GLfloat lightAmbientColor[] = { 0.125f, 0.125f, 0.125f, 0.0 };
	GLfloat lightPoition[] = { 500.0f, 200.0f, 500.0f, 0.0 };
	
	glLightfv(GL_LIGHT0, GL_POSITION, lightPoition);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbientColor);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	//enable depth testing
    glEnable(GL_DEPTH_TEST);
	glEnable( GL_CULL_FACE );
}

int tanks = 0;
NewtonBody *tankBody[2];
NTTank *nttanks[2]={0,0};

- (void)initScene
{
	NewtonDestroyAllBodies(nWorld);
	tanks = 0;
	
	// create the the floor collision, and body with default values
	NewtonCollision *collision = NewtonCreateBox (nWorld, 100, 2, 100, NULL); 
	NewtonBody *floorBody = NewtonCreateBody (nWorld, collision);
	NewtonReleaseCollision (nWorld, collision);
	
	// set the transformation for this rigid body
	float floorMatrix[16]={1,0,0,0,
					  0,1,0,0,
					  0,0,1,0,
					  0,-1,0,1};
	NewtonBodySetMatrix (floorBody, floorMatrix);


	if(boxes) [boxes release];
	boxes = [[NSMutableArray alloc] init];
	
	// create the collsion shape
	collision = NewtonCreateBox (nWorld, 0.5f, 0.5f, 0.5f, NULL); 
	
	[self createBoxStackAtX:1 atY:1 height:playerBoxes[0]+1];
	[self createBoxStackAtX:0 atY:0 height:playerBoxes[1]+1];
}

-(void) createBoxStackAtX:(int)x atY:(int)y height:(int)z
{
	int i, k, j;
	// create the collsion shape
	NewtonCollision *collision = NewtonCreateBox (nWorld, 0.5f, 0.5f, 0.5f, NULL); 
	NewtonCollision *tankcollision = NewtonCreateBox (nWorld, 0.5f, 0.25f, 0.5f, NULL); 
	
		for (j = 0; j < z; j ++)
		{ 
				// create the ridid body
				NewtonBody *boxBody ;
				
				// save the pointer to the graphic object with the body.
				NTBox *box;
				if(j<z-1)
				{
					boxBody = NewtonCreateBody (nWorld, collision);
					box = [[NTBox alloc] init];
					// set the force and torque call back funtion
					NewtonBodySetForceAndTorqueCallback (boxBody, PhysicsApplyForceAndTorque);
				}
				else
				{
					boxBody = NewtonCreateBody (nWorld, tankcollision);
					tankBody[tanks] = boxBody;
					if(!nttanks[tanks])
						nttanks[tanks] = [[NTTank alloc] init];
					box = nttanks[tanks];
					tanks++;
					
					NewtonBodySetAutoFreeze (boxBody, 0);
					NewtonWorldUnfreezeBody (nWorld, boxBody);
					// set the force and torque call back funtion
					NewtonBodySetForceAndTorqueCallback (boxBody, TankApplyForceAndTorque);
				}
				NewtonBodySetUserData (boxBody, box);
				[boxes addObject:box];

				// set the body mass and inertia
				NewtonBodySetMassMatrix (boxBody, 1.0f, 1.0f, 1.0f, 1.0f);

				// set the trasformation matrix
				float matrix[16]={1,0,0,0,
								  0,1,0,0,
								  0,0,1,0,
								  x*2,j,y*2,1};
				NewtonBodySetMatrix (boxBody, matrix);
				[box setMatrix:matrix];
				
				// set the tranform call back function
				NewtonBodySetTransformCallback (boxBody, PhysicsSetTransform);

			}
	// release the collsion geometry when not need it
	NewtonReleaseCollision (nWorld, collision);
}

- (void)dealloc
{
	// destroy the Newton world
	NewtonDestroy (nWorld);
	[boxes release];
}

#pragma mark Callbacks

// set the tranformation of a rigid body
void  PhysicsApplyForceAndTorque (const NewtonBody* body)
{
	float mass, Ixx, Iyy, Izz;

	NewtonBodyGetMassMatrix (body, &mass, &Ixx, &Iyy, &Izz);
	float force[4] = {0.0f, -mass * 9.8f, 0.0f};
	NewtonBodyAddForce (body, force);
}

float forceApplied[3];

float shootGauge = 0;
NSTimeInterval diff;
// set the tranformation of a rigid body
void  TankApplyForceAndTorque (const NewtonBody* body)
{
	float mass, Ixx, Iyy, Izz;
	float matrix[16];
	NewtonBodyGetMatrix(body, matrix);
	NewtonBodyGetMassMatrix (body, &mass, &Ixx, &Iyy, &Izz);
	
	float force[4] = {0.0f, -mass * 9.8f, 0.0f};
	NewtonBodyAddForce (body, force);
	
	if(body != tankBody[player-1]) return;
	
	if(gRight)
	{	
		float force[3] = {0.0f, -10, 0.0f};
		NewtonBodyAddTorque (body, force);
	}
	if(gLeft)
	{	
		float force[3] = {0.0f, 10, 0.0f};
		NewtonBodyAddTorque (body, force);
	}
	float speed = 10;
	if(gUp)
	{
		float force[3] = {-speed*(matrix[0]), -speed*(matrix[4]), speed*(matrix[8])};

		forceApplied[0] = force[0];
		forceApplied[1] = force[1];
		forceApplied[2] = force[2];
		NewtonBodyAddForce (body, force);
	}
	if(gDown)
	{	
		float force[3] = {speed*matrix[0], speed*matrix[4], -speed*matrix[8]};
		NewtonBodyAddForce (body, force);
	}
	
	if(gShoot) shootGauge += diff*4;
	
	if(!gShoot && shootGauge > 0)
	{
		NewtonBody* sphBody;
		NewtonCollision* collision;

		float radius = 0.1f;
		// create the collision 
		collision = NewtonCreateSphere (nWorld, radius, radius, radius, NULL); 
			
		//create the rigid body
		sphBody = NewtonCreateBody (nWorld, collision);
		
		// set the mass matrix
		NewtonBodySetMassMatrix (sphBody, 0.5f, 0.5f, 0.5, 0.5);
		
		NTBall *box = [[NTBall alloc] init];
		NewtonBodySetUserData (sphBody, box);
		[boxes addObject:box];
		
		float matrix[16];
		NewtonBodyGetMatrix(tankBody[player-1], matrix);
							
		float force[3] = {-shootGauge*matrix[0], -shootGauge*matrix[4], shootGauge*matrix[8]};
		
		float len = sqrt(force[0]*force[0]+force[1]*force[1]+force[2]*force[2]);
		
		matrix[12] += 0.35 * force[0] / len ;
		//matrix[13] += 1 ;
		matrix[14] += 0.35 * force[2] / len ;
		
		NewtonBodySetMatrix (sphBody, matrix);
		[box setMatrix:matrix];
		
		NSLog(@"make ball %f", shootGauge);
		

				// set the tranform call back function
				NewtonBodySetTransformCallback (sphBody, PhysicsSetTransform);
				// set the force and torque call back funtion
				NewtonBodySetForceAndTorqueCallback (sphBody, PhysicsApplyForceAndTorque);
		
		
		NewtonBodySetVelocity(sphBody, force);
		force[0] *= -15;
		force[1] *= -15;
		force[2] *= -15;
		NewtonBodyAddForce (body, force);
		shootGauge = 0;
		
		NewtonReleaseCollision (nWorld, collision);
		
		player++; if(player > 2) player = 1;
	}
}

// add force and torque to rigid body
void  PhysicsSetTransform (const NewtonBody* body, const float* matrix)
{
	NTBox *primitive;

	// get the graphic object form the rigid body
	primitive = (NTBox*) NewtonBodyGetUserData (body);

	// set the transformation matrix for this rigid body
	[primitive setMatrix:matrix];
}

#pragma mark Drawing

- (void) doTimeStep:(id)sender
{
	[self setNeedsDisplay:YES];
}

- (void)reshape
{
	NSRect bounds = [self bounds];
	
	glViewport(0, 0, (GLsizei) bounds.size.width, (GLsizei) bounds.size.height);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, (GLfloat) bounds.size.width/(GLfloat) bounds.size.height, 1.0, 30.0);
	glTranslatef(0.0, -2.0, -5);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

#define NEWTON_TIMESTEP 0.01

NSTimeInterval real_time = 0;
NSTimeInterval world_time = 0;

- (void)physicsThread:(id)info
{
	NSAutoreleasePool *arpool = [[NSAutoreleasePool alloc] init];
	while(YES) [self physics];
	[arpool release];
}

- (void)physics
{
	// time slice
	static NSTimeInterval accumulated_time = 0;
	
	if(world_time == 0)
		world_time = [NSDate timeIntervalSinceReferenceDate];
		
	//NSLog(@"%f",diff);
	
	// updtate the Newton physics world
	//while(world_time < [NSDate timeIntervalSinceReferenceDate])
	diff = [NSDate timeIntervalSinceReferenceDate] - world_time;
	{ 
		NewtonUpdate( nWorld, diff);//NEWTON_TIMESTEP);
		world_time = [NSDate timeIntervalSinceReferenceDate];//2*NEWTON_TIMESTEP;
	}
	
	float matrix[16];
	NewtonBodyGetMatrix(tankBody[0], matrix);
	if(matrix[13] < 0.3)
	{
		playerBoxes[1]++;
		playerBoxes[0]--;
		[view initScene];
		return;
	}
	NewtonBodyGetMatrix(tankBody[1], matrix);
	if(matrix[13] < 0.3)
	{
		playerBoxes[0]++;
		playerBoxes[1]--;
		[view initScene];
		return;
	}
	//if(mp)
	//[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:NEWTON_TIMESTEP-([NSDate timeIntervalSinceReferenceDate]-world_time)]];
}

- (void)drawRect:(NSRect)r
{

    NSRect bounds = [self bounds];
    [[NSColor clearColor] set];
    NSRectFill(bounds);
		
	[infoText setStringValue:[NSString stringWithFormat:@"Player %d", player]];

	[self physics];
	
	//camera position
	
	float matrix[16];
	NewtonBodyGetMatrix(tankBody[player-1], matrix);
						
	float camera[3] = {4*matrix[0]+matrix[12], 4+matrix[13], -4*matrix[8]+matrix[14]};
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, (GLfloat) bounds.size.width/(GLfloat) bounds.size.height, 1.0, 30.0);
	gluLookAt(camera[0],camera[1],camera[2],  matrix[12], matrix[13], matrix[14],  0,1,0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// render the object
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
/*	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glColor3f(0,1,0);
	glTranslatef(matrix[12],matrix[13],matrix[14]);
	glBegin(GL_LINES);
	glVertex3f(0,0,0);
	glVertex3f(-5*matrix[0],-5*matrix[4],5*matrix[8]);
	glEnd();
	glColor3f(0,0,1);
	glBegin(GL_LINES);
	glVertex3f(0,0,0);
	glVertex3f(5*forceApplied[0],5*forceApplied[1],5*forceApplied[2]);
	glEnd();
	glTranslatef(-matrix[12],-matrix[13],-matrix[14]);
	glColor3f(1,1,1);*/
	glColor3f(1,1,1);
	
	//glDisable(GL_LIGHTING);
	glPushMatrix();
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glBindTexture(GL_TEXTURE_2D, floorTex);
	float white[3]={1,1,1};
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, white);
	glColor3f(1,1,1);
	glScalef(10,10,10);
	glBegin(GL_QUADS);
	glColor3f(1,1,1);
	// top
	glNormal3f(0.0, 20.0, 0.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(-1,  0,  1);
	glTexCoord2f(1.0, 0.0); glVertex3f( 1,  0,  1);
	glTexCoord2f(1.0, 1.0); glVertex3f( 1,  0, -1);
	glTexCoord2f(0.0, 1.0); glVertex3f(-1,  0, -1);
	glEnd();
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	[boxes makeObjectsPerformSelector:@selector(draw)];
	
	[OBJModel startShadows];
	[boxes makeObjectsPerformSelector:@selector(drawShadow)];
	[OBJModel finishShadows];

	glFlush();
}

#pragma mark Interaction

- (BOOL)canBecomeKeyView
{
	return YES;
}
- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (void) keyDown:(NSEvent *)theEvent
{
   unichar unicodeKey = [ [ theEvent characters ] characterAtIndex:0 ];
   
   switch( unicodeKey )
   {
      case '8':
      case 'i':
      case 'w':
	  case NSUpArrowFunctionKey:
		gUp=YES;
		break;
      case '5':
      case 'k':
      case 's':
	  case NSDownArrowFunctionKey:
		gDown=YES;
		break;
      case '4':
      case 'j':
      case 'a':
	  case NSLeftArrowFunctionKey:
		gLeft=YES;
		break;
      case '6':
      case 'l':
      case 'd':
	  case NSRightArrowFunctionKey:
		gRight=YES;
		break;
      case ' ':
		gShoot=YES;
		break;
   }
}

- (void) keyUp:(NSEvent *)theEvent
{
   unichar unicodeKey = [ [ theEvent characters ] characterAtIndex:0 ];
   
   switch( unicodeKey )
   {
      case '8':
      case 'i':
      case 'w':
	  case NSUpArrowFunctionKey:
		gUp=NO;
		break;
      case '5':
      case 'k':
      case 's':
	  case NSDownArrowFunctionKey:
		gDown=NO;
		break;
      case '4':
      case 'l':
      case 'a':
	  case NSLeftArrowFunctionKey:
		gLeft=NO;
		break;
      case '6':
      case 'j':
      case 'd':
	  case NSRightArrowFunctionKey:
		gRight=NO;
		break;
      case ' ':
		gShoot=NO;
		break;
   }
}

@end
